#!/usr/bin/env python3

import netimpact.processor as P
from netimpact.awin import AWin
from netimpact.admitad import Admitad
from netimpact.linkshare import Linkshare
from netimpact.impact import Impact #, Clicker
import click
import toml
import boto3
from s3fs import S3FileSystem
import pandas as pd

import logging,os,time
from datetime import datetime, timedelta, date
logging.basicConfig(level=os.environ.get("LOGLEVEL", "INFO"))

@click.argument('networks')
@click.option('--config', '-c', default='config.toml', help='relative filepath to the TOML config file (see README for formatting requirements). Defaults to looking for a config.toml file in the current directory.')
@click.option('--partners', '-p', is_flag=True, default=False, help='get and upload new partners?')
@click.option('--transactions', '-t', is_flag=True, default=False, help='get and upload transactions?')
@click.option('--groups', '-g', is_flag=True, default=False, help='refresh group list?')
@click.option('--no_upload', is_flag=True, default=False, help='If flag is included then transactions will be pulled from the listed networks and put in the transactions folder locally instead of uploaded to Impact.')
@click.option('--s3_upload', '-s', default=False, help='Name of the S3 bucket to upload data to. References ~/.aws/credentials')
@click.option('--target_date', '-d', type=click.DateTime(formats=["%Y-%m-%d"]),
              default=str(date.today()-timedelta(1)), help='Transactions from what day? Format: %Y-%m-%d')
@click.command()
def cli(networks,config,partners,transactions,groups,no_upload,s3_upload,target_date):
    """Default CLI method to get new partners and transactions from the
    provided NETWORKS and create them in the Impact program.

    NETWORKS is the comma seperated list of networks to operate on (currently accepts awin,linkshare,admitad)
    """
    c = toml.load(config)
    logging.info(f'Starting import process for {networks}')
    i = Impact(
        c['Impact']['impact_sid'],
        c['Impact']['impact_token'],
        c['Impact']['ftp_un'],
        c['Impact']['ftp_p'],
        c['Impact']['program_id'],
        c['Impact']['desktop_action_tracker_id'],
        c['Impact']['mobile_action_tracker_id'],
        reload_groups=groups
        )

    network_list = []
    network_names = []
    if 'awin' in networks.lower():
        network_list.append(AWin(c['Awin']['oauth']))
        network_names.append('awin')

    if 'admitad' in networks.lower():
        network_list.append(Admitad(c['Admitad']['client_id'], c['Admitad']['client_secret']))
        network_names.append('admitad')
    
    if 'ls' in networks.lower() or 'linkshare' in networks.lower() or 'rakuten' in networks.lower():
        network_list.append(Linkshare(c['Linkshare']['report_names']['Transactions'], c['Linkshare']['report_names']['Publishers']))
        network_names.append('linkshare')

    for n in network_list:
        logging.info(f'Running data import for {n.network_name} into Impact program {i.program_id}')
        for account_name,account_id in c[n.network_name]['account_ids'].items():
            if account_name not in i.groups_dict.keys():
                i.groups_dict[account_name] = i.create_group(account_name)
            if partners:
                i.new_publisher_validation(account_id, account_name, n)
            if transactions:
                file_path_m, file_path_p = P.transactions_process(i, account_id, account_name, n, target_date)
                if no_upload:
                    pass
                else:
                    i.batch_to_impact(file_path_m, file_path_p)
                if s3_upload:
                    session = boto3.Session(
                        aws_access_key_id=c['S3']['access_key'],
                        aws_secret_access_key==c['S3']['secret_access_key'],
                    )
                    s3 = boto3.resource('s3')
                    df = pd.read_csv(file_path_p)
                    pq_file = f'{str(file_path_p).rstrip(".csv")}.parquet'
                    df.to_parquet(pq_file)
                    s3.Object(c['S3']['bucket'], pq_file).put(Body=open(pq_file, 'rb'))
                    s3.Object(c['S3']['bucket'], str(file_path_m)).put(Body=open(str(file_path_m), 'rb'))


import requests
import http.client
import mimetypes
import json
from urllib.parse import urlencode
from base64 import b64encode
from datetime import timedelta

class Admitad:
    """Admitad object for interacting with the Admitad advertiser API

    Arguments:
        client_id {string} -- Admitad client ID
        client_secret {string} -- Admitad client secret

    Keyword Arguments:
        token {str} -- If provided this string is used instead of generating a new token (default: {False})
    """    
    network_name = 'Admitad'
    date_format = '%d.%m.%Y'

    def __init__(self, client_id, client_secret, token=False):    
        super().__init__()
        self.client_id = client_id
        self.client_secret = client_secret
        if token:
            self.headers = {'Authorization': f'Bearer {token}'}
        else:
            self.headers = {'Authorization': f'Bearer {self.get_token()}'}

    def get_token(self):
        """Generate authorisation token from Admitad

        Returns:
            string -- Admitad access token
        """        
        auth = b64encode(bytes((self.client_id + ':' + self.client_secret),'utf-8'))

        conn = http.client.HTTPSConnection("api.admitad.com")
        payload = ''

        headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': f'Basic {auth.decode("utf-8")}'
        }
        data = {
            'grant_type': 'client_credentials',
            'client_id': self.client_id,
            'scope': 'advertiser_websites advertiser_statistics advertiser_info'
         }
        encoded_args = urlencode(data)
        conn.request("POST", f"/token/?{encoded_args}", payload, headers)
        res = conn.getresponse()
        data = res.read()
        return json.loads(data)['access_token']


    def get_pubs(self, acct):
        """GET the list of websites connected to the Admitad program as publishers and extract the relevant partner information

        Arguments:
            acct {string} -- Admitad account ID

        Returns:
            list -- A list of dictionaries containing publisher name, id, and site url
        """        
        conn = http.client.HTTPSConnection("api.admitad.com")
        payload = ''
        conn.request("GET", f"/advertiser/{acct}/websites/?limit=1000", payload, self.headers)
        res = conn.getresponse()
        data = res.read()
        admitad_sites = json.loads(data.decode("utf-8"))['results']
        pub_list = []
        for ap in admitad_sites:
            pub = {
                'name': f"{ap['website']['name']} - {ap['user']['name']}",
                'id': ap['website']['id'],
                'site': ap['website']['site_url']
            }
            pub_list.append(pub)
        return pub_list


    def transaction_request(self, acct, start, end, status):
        """GET all transactions over the given time period with a given status and transform
        the transactions into dictionaries consumable by the Impact transaction upload object

        Arguments:
            acct {string} -- Admitad account ID
            start {datetime} -- Datetime object for the beginning of the transaction time period
            end {datetime} -- Datetime object for the end of the transaction time period
            status {string} -- Status of the transactions to be pulled - one of [pending, approved, declined]. Defaults to pending if incorrect input is given.


        Returns:
            dict -- Dictionary containing transaction data
        """
        if status == 'pending':
            st = 0
        elif status == 'approved':
            st = 1
        elif status == 'declined':
            st = 2
        else:
            st = 0
        conn = http.client.HTTPSConnection("api.admitad.com")
        payload = ''

        
        conn.request("GET", f"/advertiser/{acct}/statistics/actions/?start_date={start}&end_date={end}&status={st}&limit=5000", payload, self.headers)
        res = conn.getresponse()
        data = res.read()
        return json.loads(data.decode("utf-8"))['results']

    def get_all_transactions(self, acct, start, end, status):
        """GET all transactions over the given time period with a given status and transform
        the transactions into dictionaries consumable by the Impact transaction upload object
        TODO :: Add flexibility for the identifier of new vs returning and device type

        Arguments:
            acct {string} -- Admitad account ID
            start {datetime} -- Datetime object for the beginning of the transaction time period
            end {datetime} -- Datetime object for the end of the transaction time period
            status {string} -- Status of the transactions to be pulled - one of [pending, approved, declined]. Defaults to pending if incorrect input is given.

        Returns:
            list -- List of dictionaries containing transaction data
        """        
        transactions = self.transaction_request(acct,start,end,status)
        transaction_list = []
        for t in transactions:
            try:
                if t['product_id'] == 1636:
                    nvr = 'New'
                else:
                    nvr = 'Returning'
            except KeyError as e:
                nvr = 'Returning'
            transaction = {
                'transactionDate': t['action_time'],
                'id': t['id'],
                'saleAmount': {'amount':t['order_sum'], 'currency':t['currency']},
                'commissionAmount': {'amount':t['payment_webmaster'], 'currency':t['currency']},
                'publisherId': t['website_id'],
                'status': nvr,
                'voucherCode': t['promocode'],
                'customerCountry': t['action_country'],
                'advertiserCountry': t['action_country']
            }
            try:
                if 'мобильный' in t['product_name']:
                    transaction['device'] = 'Mobile'
                else:
                    transaction['device'] = 'Desktop'
                transaction_list.append(transaction)
            except TypeError as e:
                transaction['device'] = 'Desktop'
        return transaction_list

    def date_formatter(self, target_date, target_start_date=False):
        if target_start_date:
            pass
        else:
            start = (target_date - timedelta(1)).strftime(self.date_format)
        end = target_date.strftime(self.date_format)
        return start, end


class Network:
    network_name = 'NETWORK'
    def __init__(self):
        '''
        Pass network API credentials/secrets/tokens to the object. Use them in the two required functions below to get publishers and transactions
        '''
        pass

    def get_pubs(self, acct):
        publishers = [{'name':'example','id':'example_id','site_url':'example_url'}] # Replace this with whatever network call needs to happen to get a pub list
        pub_list = []
        for p in publishers:
            pub = {
                'name': {p['name']},
                'id': p['id'],
                'site': p['site_url']
            }
            pub_list.append(pub)
        return pub_list

    def get_all_transactions(self, acct, start, end, status):
        transactions = [{'name':'example','id':'example_id','site_url':'example_url'}] # Replace this with whatever network call needs to happen to get a pub list

        transaction_list = []
        for t in transactions:
            transaction = {
                    'transactionDate': t['action_time'],
                    'id': t['id'],
                    'saleAmount': {'amount':t['order_total'], 'currency':t['order_currency']},
                    'commissionAmount': {'amount':t['commission'], 'currency':t['commission_currency']},
                    'publisherId': t['website_id'],
                    'status': t['customer_status'],
                    'voucherCode': t['promocode'],
                    'customerCountry': t['action_country'],
                    'advertiserCountry': t['action_country']
                }
            transaction_list.append(transaction)
        return transaction_list




import requests
import csv
import logging, os
import json
from datetime import datetime, timedelta

class Linkshare:
    """Linkshare object for interacting with the Linkshare advertiser API"""
    network_name = 'Linkshare'
    date_format = '%Y-%m-%d'

    countries = {"BD": "Bangladesh", "BE": "Belgium", "BF": "Burkina Faso", "BG": "Bulgaria", "BA": "Bosnia and Herzegovina", "BB": "Barbados", "WF": "Wallis and Futuna", "BL": "Saint Barthelemy", "BM": "Bermuda", "BN": "Brunei", "BO": "Bolivia", "BH": "Bahrain", "BI": "Burundi", "BJ": "Benin", "BT": "Bhutan", "JM": "Jamaica", "BV": "Bouvet Island", "BW": "Botswana", "WS": "Samoa", "BQ": "Bonaire, Saint Eustatius and Saba ", "BR": "Brazil", "BS": "Bahamas", "JE": "Jersey", "BY": "Belarus", "BZ": "Belize", "RU": "Russia", "RW": "Rwanda", "RS": "Serbia", "TL": "East Timor", "RE": "Reunion", "TM": "Turkmenistan", "TJ": "Tajikistan", "RO": "Romania", "TK": "Tokelau", "GW": "Guinea-Bissau", "GU": "Guam", "GT": "Guatemala", "GS": "South Georgia and the South Sandwich Islands", "GR": "Greece", "GQ": "Equatorial Guinea", "GP": "Guadeloupe", "JP": "Japan", "GY": "Guyana", "GG": "Guernsey", "GF": "French Guiana", "GE": "Georgia", "GD": "Grenada", "UK": "United Kingdom", "GA": "Gabon", "SV": "El Salvador", "GN": "Guinea", "GM": "Gambia", "GL": "Greenland", "GI": "Gibraltar", "GH": "Ghana", "OM": "Oman", "TN": "Tunisia", "JO": "Jordan", "HR": "Croatia", "HT": "Haiti", "HU": "Hungary", "HK": "Hong Kong", "HN": "Honduras", "HM": "Heard Island and McDonald Islands", "VE": "Venezuela", "PR": "Puerto Rico", "PS": "Palestinian Territory", "PW": "Palau", "PT": "Portugal", "SJ": "Svalbard and Jan Mayen", "PY": "Paraguay", "IQ": "Iraq", "PA": "Panama", "PF": "French Polynesia", "PG": "Papua New Guinea", "PE": "Peru", "PK": "Pakistan", "PH": "Philippines", "PN": "Pitcairn", "PL": "Poland", "PM": "Saint Pierre and Miquelon", "ZM": "Zambia", "EH": "Western Sahara", "EE": "Estonia", "EG": "Egypt", "ZA": "South Africa", "EC": "Ecuador", "IT": "Italy", "VN": "Vietnam", "SB": "Solomon Islands", "ET": "Ethiopia", "SO": "Somalia", "ZW": "Zimbabwe", "SA": "Saudi Arabia", "ES": "Spain", "ER": "Eritrea", "ME": "Montenegro", "MD": "Moldova", "MG": "Madagascar", "MF": "Saint Martin", "MA": "Morocco", "MC": "Monaco", "UZ": "Uzbekistan", "MM": "Myanmar", "ML": "Mali", "MO": "Macao", "MN": "Mongolia", "MH": "Marshall Islands", "MK": "Macedonia", "MU": "Mauritius", "MT": "Malta", "MW": "Malawi", "MV": "Maldives", "MQ": "Martinique", "MP": "Northern Mariana Islands", "MS": "Montserrat", "MR": "Mauritania", "IM": "Isle of Man", "UG": "Uganda", "TZ": "Tanzania", "MY": "Malaysia", "MX": "Mexico", "IL": "Israel", "FR": "France", "IO": "British Indian Ocean Territory", "SH": "Saint Helena", "FI": "Finland", "FJ": "Fiji", "FK": "Falkland Islands", "FM": "Micronesia", "FO": "Faroe Islands", "NI": "Nicaragua", "NL": "Netherlands", "NO": "Norway", "NA": "Namibia", "VU": "Vanuatu", "NC": "New Caledonia", "NE": "Niger", "NF": "Norfolk Island", "NG": "Nigeria", "NZ": "New Zealand", "NP": "Nepal", "NR": "Nauru", "NU": "Niue", "CK": "Cook Islands", "XK": "Kosovo", "CI": "Ivory Coast", "CH": "Switzerland", "CO": "Colombia", "CN": "China", "CM": "Cameroon", "CL": "Chile", "CC": "Cocos Islands", "CA": "Canada", "CG": "Republic of the Congo", "CF": "Central African Republic", "CD": "Democratic Republic of the Congo", "CZ": "Czech Republic", "CY": "Cyprus", "CX": "Christmas Island", "CR": "Costa Rica", "CW": "Curacao", "CV": "Cape Verde", "CU": "Cuba", "SZ": "Swaziland", "SY": "Syria", "SX": "Sint Maarten", "KG": "Kyrgyzstan", "KE": "Kenya", "SS": "South Sudan", "SR": "Suriname", "KI": "Kiribati", "KH": "Cambodia", "KN": "Saint Kitts and Nevis", "KM": "Comoros", "ST": "Sao Tome and Principe", "SK": "Slovakia", "KR": "South Korea", "SI": "Slovenia", "KP": "North Korea", "KW": "Kuwait", "SN": "Senegal", "SM": "San Marino", "SL": "Sierra Leone", "SC": "Seychelles", "KZ": "Kazakhstan", "KY": "Cayman Islands", "SG": "Singapore", "SE": "Sweden", "SD": "Sudan", "DO": "Dominican Republic", "DM": "Dominica", "DJ": "Djibouti", "DK": "Denmark", "VG": "British Virgin Islands", "DE": "Germany", "YE": "Yemen", "DZ": "Algeria", "US": "United States", "UY": "Uruguay", "YT": "Mayotte", "UM": "United States Minor Outlying Islands", "LB": "Lebanon", "LC": "Saint Lucia", "LA": "Laos", "TV": "Tuvalu", "TW": "Taiwan", "TT": "Trinidad and Tobago", "TR": "Turkey", "LK": "Sri Lanka", "LI": "Liechtenstein", "LV": "Latvia", "TO": "Tonga", "LT": "Lithuania", "LU": "Luxembourg", "LR": "Liberia", "LS": "Lesotho", "TH": "Thailand", "TF": "French Southern Territories", "TG": "Togo", "TD": "Chad", "TC": "Turks and Caicos Islands", "LY": "Libya", "VA": "Vatican", "VC": "Saint Vincent and the Grenadines", "AE": "United Arab Emirates", "AD": "Andorra", "AG": "Antigua and Barbuda", "AF": "Afghanistan", "AI": "Anguilla", "VI": "U.S. Virgin Islands", "IS": "Iceland", "IR": "Iran", "AM": "Armenia", "AL": "Albania", "AO": "Angola", "AQ": "Antarctica", "AS": "American Samoa", "AR": "Argentina", "AU": "Australia", "AT": "Austria", "AW": "Aruba", "IN": "India", "AX": "Aland Islands", "AZ": "Azerbaijan", "IE": "Ireland", "ID": "Indonesia", "UA": "Ukraine", "QA": "Qatar", "MZ": "Mozambique"}
    def __init__(self, transaction_report_name, pub_report_name):
        self.transaction_report_name = transaction_report_name
        self.pub_report_name = pub_report_name

    def get_pubs(self, acct_token):
        """GET the list of websites connected to the Linkshare program as publishers and extract the relevant partner information

        Arguments:
            acct_token {string} -- Linkshare account specific token

        Returns:
            list -- A list of dictionaries containing publisher name, id, and site url
        """    
        pub_list = []
        url = f'https://ran-reporting.rakutenmarketing.com/en/reports/{self.pub_report_name}/filters?'
        params = {
            'date_range': 'yesterday',
            'include_summary': 'N',
            'tz': 'GMT',
            'date_type':'transaction',
            'token': acct_token
        }
        r = requests.get(url, params=params)
        if r.status_code:
            data = r.content.decode('utf-8')
            cr = csv.reader(data.splitlines(), delimiter=',')
            results_list = list(cr)
            for lsp in results_list[1:]:
                pub = {
                    'name': lsp[4],
                    'id': lsp[3],
                    'site': lsp[5]
                }
                
                pub_list.append(pub)
            return pub_list
        else:
            logging.error('Linkshare Publisher Report request error')
            logging.error(r.text)
            return []

    def get_all_transactions(self, acct, start, end):
        """GET all transactions over the given time period with a given status and transform the transactions into dictionaries consumable by the Impact transaction upload object
        TODO :: Add flexibility for the identifier of new vs returning and device type

        Arguments:
            acct {string} -- Rakuten account token
            start {datetime} -- Datetime object for the beginning of the transaction time period
            end {datetime} -- Datetime object for the end of the transaction time period

        Returns:
            list -- List of dictionaries containing transaction data
        """       
        transactions = [{'name':'example','id':'example_id','site_url':'example_url'}] # Replace this with whatever network call needs to happen to get a pub list
        url = f'https://ran-reporting.rakutenmarketing.com/en/reports/{self.transaction_report_name}/filters?'
        params = {
            'start_date': start,
            'end_date': end,
            'include_summary': 'N',
            'tz': 'GMT',
            'date_type': 'transaction',
            'token': acct
            }
        r = requests.get(url, params=params)
        if r.status_code:
            data = r.content.decode('utf-8')
            cr = csv.reader(data.splitlines(), delimiter=',')
            transactions = list(cr)
        pending = []
        approved = []
        declined = []
        expected = ['\ufeffConsumer Country','Gross Commissions','Gross Sales','Order ID','Publisher ID','Transaction Date','Transaction Time','Process Date','Customer Status', 'Currency']
        headers = transactions[0]
        for e in expected:
            if e not in headers:
                logging.error(f'Linkshare report does not contain correct header - did not find {e}')
                logging.warn(headers)
                return [], [], []
        for t in transactions[1:]:
            country = ''
            for k,v in self.countries.items():
                if t[headers.index('\ufeffConsumer Country')] in v:
                    country = k
            transaction = {
                    'id': t[headers.index('Order ID')],
                    'saleAmount': {'amount':float(t[headers.index('Gross Sales')].replace(',','')), 'currency':t[headers.index('Currency')]},
                    'commissionAmount': {'amount':t[headers.index('Gross Commissions')].replace(',',''), 'currency':t[headers.index('Currency')]},
                    'publisherId': t[headers.index('Publisher ID')],
                    'voucherCode': '',
                    'customerCountry': country,
                    'advertiserCountry': country,
                    'device' : 'Desktop'
                }
            if t[headers.index('Customer Status')] == 'New':
                transaction['status'] = 'New'
            else:
                transaction['status'] = 'Returning'
            date_components = t[headers.index('Transaction Date')].split('/')
            for v in date_components:
                if len(v) < 2:
                    date_components[date_components.index(v)] = f'0{v}'
            timing = t[headers.index('Transaction Time')]
            transaction['transactionDate'] = f'20{date_components[2]}-{date_components[0]}-{date_components[1]}T{timing}'
            
            if (t[headers.index('Transaction Date')] == t[headers.index('Process Date')]) and (transaction['saleAmount']['amount'] > 0.0):
                pending.append(transaction)
            else:
                if float(transaction['commissionAmount']['amount']) == 0.0:
                    declined.append(transaction)
                else:
                    approved.append(transaction)


        return approved, pending, declined

    def date_formatter(self, target_date, target_start_date=False):
        if target_start_date:
            pass
        else:
            start = (target_date - timedelta(1)).strftime(self.date_format)
        end = (target_date).strftime(self.date_format)
        return start, end


#!/usr/bin/env python3
import requests
import logging
import time
from datetime import timedelta, datetime

class AWin:
    """AWin object for interacting with the AWin advertiser API

    Arguments:
        token {string} -- AWin API auth token
    """    
    network_name = 'Awin'
    date_format = '%Y-%m-%d'

    def __init__(self, token):    
        self.token = token

    def get_pubs(self, acct):
        """GET the list of websites connected to the Awin program as publishers and extract the relevant partner information

        Arguments:
            acct {string} -- Awin account ID

        Returns:
            list -- A list of dictionaries containing publisher name, id, and site url
        """         
        url = f"https://api.awin.com/advertisers/{acct}/publishers?&accessToken={self.token}"
        r = requests.get(url)
        while r.status_code == 429:
            time.sleep(60)
            r = requests.get(url)
        pub_list = r.json()
        for p in pub_list:
            p['site'] = 'https://www.awinpub.com'
        return pub_list

    def transaction_request(self, acct, start, end, status):
        """GET all transactions over the given time period with a given status and transform
        the transactions into dictionaries consumable by the Impact transaction upload object

        Arguments:
            acct {string} -- Awin account ID
            start {datetime} -- Datetime object for the beginning of the transaction time period
            end {datetime} -- Datetime object for the end of the transaction time period
            status {string} -- Status of the transactions to be pulled - one of [pending, approved, declined]. Defaults to pending if incorrect input is given.


        Returns:
            dict -- Dictionary containing transaction data
        """
        url = f"https://api.awin.com/advertisers/{acct}/transactions/?startDate={start}T00%3A00%3A00&endDate={end}T23%3A59%3A59&timezone=GMT&accessToken={self.token}&status={status}"
        if status == 'approved' or status == 'declined':
            url += '&dateType=validation'
        r = requests.get(url)
        while r.status_code == 429:
            time.sleep(60)
            r = requests.get(url)
        return r.json()

    def get_all_transactions(self, acct, start, end, status):
        """GET all transactions over the given time period with a given status and transform
        the transactions into dictionaries consumable by the Impact transaction upload object
        TODO :: Add flexibility for the identifier of new vs returning and device type

        Arguments:
            acct {string} -- Awin account ID
            start {datetime} -- Datetime object for the beginning of the transaction time period
            end {datetime} -- Datetime object for the end of the transaction time period
            status {string} -- Status of the transactions to be pulled - one of [pending, approved, declined]. Defaults to pending if incorrect input is given.


        Returns:
            list -- List of dictionaries containing transaction data
        """     

        transaction_list = self.transaction_request(acct, start, end, status)
        for t in transaction_list:
            if status == 'pending':
                try:
                    for cg in t['transactionParts']:
                        if 'New Customer' in cg['commissionGroupName']:
                            t['status'] = 'New'
                        else:
                            t['status'] = 'Returning'
                except KeyError as e:
                    logging.warn(f'No CG on transaction {t["id"]}')
                    t['status'] = 'Returning'
            if 'Mobile' in t['type']:
                t['device'] = 'Mobile'
            else:
                t['device'] = 'Desktop'

        return transaction_list

    def get_agg_transactions(self, acct, start, end):
        url = f"https://api.awin.com/advertisers/{acct}/reports/publisher?startDate={start}&endDate={end}&timezone=GMT&accessToken={self.token}"


    def date_formatter(self, target_date, target_start_date=False):
        if target_start_date:
            pass
        else:
            start = (target_date).strftime(self.date_format)
        end = (target_date).strftime(self.date_format)
        return start, end


from datetime import datetime, timedelta
from itertools import zip_longest
import json
import time
import os
import csv
import ftplib
from pathlib import Path
from urllib import request
import logging

import requests

class Impact:
    def __init__(self,
                sid,token,
                ftp_un,
                ftp_p,
                program_id,
                desktop_action_tracker_id,
                mobile_action_tracker_id,
                reload_groups=True):
        """Impact Partnership Cloud object for creating and modifying partners, transactions, and partner groups in a given program

        Arguments:
            sid {string} -- Impact account SID
            token {string} -- Impact account token
            ftp_un {string} -- Impact program FTP credentials
            ftp_p {string} -- Impact program FTP credentials
            program_id {string} -- Impact program ID
            desktop_action_tracker_id {string} -- Event type/action tracker ID for the desktop event
            mobile_action_tracker_id {string} -- Event type/action tracker ID for the mobile event

        Keyword Arguments:
            reload_groups {bool} -- Get group IDs for the account. Can take some time as the Impact Groups list API endpoint also lists all partners. (default: {True})
        """                   
        self.root_adv_api = f'https://{sid}:{token}@api.impact.com'
        self.adv_api_stem = f'{self.root_adv_api}/Advertisers/{sid}/'
        self.ftp_un = ftp_un
        self.ftp_p = ftp_p
        self.program_id = program_id
        self.desktop_action_tracker_id = desktop_action_tracker_id
        self.mobile_action_tracker_id = mobile_action_tracker_id
        self.io_id = self.get_contract() # Get Impact IO ID - note that this only works at the moment with a program with a single IO TODO :: Allow for IO selection on partner creation
        self.existing_partner_dict = self.get_partners()

        if reload_groups:
            self.groups_dict = self.get_groups()
        else:
            self.groups_dict = {'Linkshare UK': '17203',
                                'Linkshare US': '17204',
                                'Linkshare AU': '17205',
                                'Linkshare Asia': '17206',
                                'Linkshare Concierge': '17207',
                                'Admitad RU': '17213',
                                'Awin FR': '17214',
                                'Awin IE': '17217',
                                'Awin BE': '17218',
                                'Awin NL': '17219',
                                'Awin SE': '17225',
                                'Awin NO': '17226',
                                'Awin DK': '17227',
                                'Awin FI': '17228',
                                'Awin IT': '17229',
                                'Awin EE': '17230',
                                'Awin AU': '17231',
                                'Awin ES': '17232',
                                'Awin DE': '17233',
                                'Awin AT': '17235',
                                'Awin CH': '17236',
                                'Awin ROW': '17237',
                                'Awin UK': '17238',
                                'Awin': '17260',
                                'Linkshare': '17261',
                                'Admitad': '17262'}

    def create_group(self, group_name):
        """Creates new partner group

        Arguments:
            group_name {string} -- Group name

        Returns:
            string -- Group ID
        """        
        import requests
        url = self.adv_api_stem + f'Campaigns/{self.program_id}/MediaPartnerGroups'
        params = {
            'Name': group_name
        }
        headers = {'Content-Type': 'application/x-www-form-urlencoded', 'Accept':'application/json'}
        return requests.post(url, params=params, headers=headers).json()['Uri'].split('/')[-1]

    def create_partner(self, pub, io_id, group_name, network_name, dupe=0):
        """Create new partner attached to the program contract and tagged with the network and network program as the groups with.
        Note that the Impact partner creation API does not currently correctly assign groups on partner creation, and it returns
        a 500 error on successful creation instead of the partner ID as listed in the documentation.

        Arguments:
            pub {dict} -- Publisher dictionary
            io_id {string} -- Contract/Insertion Order ID
            group_name {string} -- Group name
            network_name {string} -- Network name

        Keyword Arguments:
            dupe {int} -- counter that determines a string to add to the end of the publisher name for uniqueness (default: {0})

        Returns:
            requests.response  -- Impact API response to the partner creation POST
        """        
        url = self.adv_api_stem + 'MediaPartners'
        publisher_account_name = pub['name'] + f' - {network_name} - AS'
        if dupe == 1:
            publisher_account_name = f'{pub["name"]} - {network_name} - {group_name.split(" ")[-1]} {randomString(2)}'
        elif dupe == 2:
            logging.warning(f'Second naming error creating partner record {pub["name"]}')
            publisher_account_name += group_name.split('_')[-1] + '2'
        params = {
            'AccountName': publisher_account_name,
            'AddressLine1': '123 Mickey Mouse Lane',
            'City': 'Disneyland',
            'PostalCode': '93101',
            'Country': 'UK',
            'PhoneCountry': 'UK',
            'PhoneNumber': '4343861194',
            'PromotionalMethods': 'CONTENT',
            'InsertionOrderId': int(io_id),
            'TaxId': '123456689',
            'OrganizationType': 'OTHER',
            'Username': randomString(10),
            'FirstName': 'Minnie',
            'LastName': 'Mouse',
            'Password': 'l3tme1n!',
            'EmailAddress': 'minnielovesmicky@gmail.com',
            'MPGroupIds': self.groups_dict[group_name],
            'MPValue1': pub["id"],
            'MPValue2': pub['name']
        }
        if  pub['site'].startswith('http'):
            params['Website'] = pub['site']
        else:
            params['Website'] = 'http://' + pub['site']
        headers = {'Content-Type': 'application/x-www-form-urlencoded', 'Accept':'application/json'}
        return requests.post(url, params=params, headers=headers)

    def get_groups(self):
        """Pull the list of groups on the program and creates a dict for easy lookup

        Returns:
            dict -- {Group Name : Group ID}
        """
        url = self.adv_api_stem + f'Campaigns/{self.program_id}/MediaPartnerGroups.json'
        r = requests.get(url)
        groups_dict = {}
        while r.status_code == 429:
            time.sleep(120)
            r = requests.get(url)
        for g in r.json()['Groups']:
            groups_dict[g['Name']] = g['Id']
        print(groups_dict)
        return groups_dict

    def add_to_group(self, group_id, partner_ids):
        """Add the list of partners to a specified group

        Arguments:
            group_id {string} -- Group ID
            partner_ids {list} -- List of partner IDs to add to the group
        """
        url = self.adv_api_stem + f'Campaigns/{self.program_id}/MediaPartnerGroups/{group_id}'
        for p in zip_longest(*(iter(partner_ids),) * 500):
            try:
                ps = ",".join(p)
            except TypeError:
                pa=''
                for a in p:
                    if a is not None:
                        pa += a + ','
                ps = pa[:-1]
            headers = {'Content-Type': 'application/x-www-form-urlencoded', 'Accept':'application/json'}
            params = {
                'MediaPartnersAdd': ps
            }
            r = requests.put(url, params=params, headers=headers)

    def get_partners(self):
        """Pull the full list of partners linked to the program and create a dict with partner IDs and existing network IDs

        Returns:
            dict -- {Network Partner ID : Impact Partner ID}
        """
        logging.debug('Refreshing full impact partner list.')
        url = self.adv_api_stem + f'MediaPartners?CampaignId={self.program_id}&PageSize=1000'
        existing_partner_dict = {}

        headers = {'Content-Type': 'application/x-www-form-urlencoded', 'Accept':'application/json'}
        while True:
            r = requests.get(url, headers=headers)
            while r.status_code == 429:
                time.sleep(30)
                r = requests.get(url, headers=headers)
            while r.status_code == 520:
                time.sleep(30)
                r = requests.get(url, headers=headers)
            rj = r.json()

            printProgressBar(int(rj['@page']), int(rj['@numpages']), prefix = f'Page {int(rj["@page"])}/{int(rj["@numpages"])}:', suffix = 'Complete', length = 50)
            time.sleep(2)
            for g in r.json()['MediaPartners']:
                if g['MPValue1'] != '':
                    existing_partner_dict[str(g['MPValue1'])] = g['Id']
            if rj['@page'] != rj['@numpages']:
                url = self.root_adv_api + rj['@nextpageuri']
            else:
                break
        return existing_partner_dict

    def get_contract(self):
        """Get Impact IO ID - note that this only works at the moment with a program with a single IO TODO :: Allow for IO selection on partner creation

        Returns:
            string -- Inserion Order/Contract ID
        """        
        url = self.adv_api_stem + f'Campaigns/{self.program_id}/Contracts.json'
        r = requests.get(url)
        while r.status_code == 429:
            time.sleep(60)
            r = requests.get(url)
        return r.json()['Contracts'][0]['TemplateTerms']['TemplateId']

    def partner_update(self, group_name, network_name, network_pubs):
        """Iterate through a list of publisher dicts and create (or try to create publishers). Get new publisher IDs in the process and add to a list to assign correct groups
        Check for new pubs, update groups. Impact's MP creation does not return the ID so if there are new partners need to re-run the entire partner lookup to find the new partners
        MPValue1 is the network partner ID

        Arguments:
            group_name {string} -- Group name (network program name)
            network_name {string} -- Network name
            network_pubs {list} -- list of dicts of publisher information of pubs to create

        Returns:
            tuple -- (Total new partners created, Total partners in program processed)
        """
        new_partners = []
        partnercount = 0
        group_add = []
        for pub in network_pubs:
            partnercount += 1
            printProgressBar(partnercount, len(network_pubs), prefix = f'Updating publisher {partnercount}/{len(network_pubs)}:', suffix = 'Complete', length = 50)
            try:
                group_add.append(self.existing_partner_dict[str(pub["id"])])
            except KeyError as e:
                logging.info(f'Creating new partner, {pub}')
                r = self.create_partner(pub, self.io_id, group_name, network_name)
                while r.status_code == 429:
                    logging.error(r.text)
                    time.sleep(300)
                    pass
                else:
                    if r.status_code == 400:
                        logging.warning(r.text)
                        r = self.create_partner(pub, self.io_id, group_name, network_name, 1)
                        if r.status_code == 400:
                            r = self.create_partner(pub, self.io_id, group_name, network_name, 2)
                        new_partners.append(pub['name'])
                    else:
                        new_partners.append(pub['name'])
        logging.info(f'Adding partners to group {self.groups_dict[group_name]}')

        self.add_to_group(self.groups_dict[group_name], group_add)
        self.add_to_group(self.groups_dict[network_name], group_add)

        return new_partners, partnercount

    def new_publisher_validation(self, account_id, account_name, network):
        """Uses the network object to get full publisher list and add new publishers to the Impact program with the correct group names (Network name, network program name)

        Arguments:
            account_id {string} -- Network specific authentication token to get publisher list
            account_name {string} -- Network program/account name
            network {object} -- Network object (currently one of AWin, Admitad, Linkshare)
        """
        network_pubs = network.get_pubs(account_id)
        logging.info(f'Updating partner list for {account_name}')
        new_partners, total_count = self.partner_update(account_name, network.network_name, network_pubs)
        message = f'New partners in {account_name}: {len(new_partners)}. Total partner count: {total_count}\n'
        logging.info(message)
        if len(new_partners) > 0:
            self.existing_partner_dict = self.get_partners()
            self.partner_update(account_name, network.network_name, network_pubs)


    def batch_to_impact(self, file_path_m, file_path_p):
        try:
            with ftplib.FTP('batch.impactradius.com', self.ftp_un, self.ftp_p) as ftp, open(file_path_p, 'rb') as file:
                ftp.storbinary(f'STOR {file_path_p.name}', file)                
                logging.info(f'{file_path_p} transactions batch uploaded.')
            os.remove(file_path_p)

        except ftplib.error_perm as e:
            logging.error(f"FTP Upload error for {file_path_p}. Re-upload this file in a few minutes.")

        try:
            with ftplib.FTP('batch.impactradius.com', self.ftp_un, self.ftp_p) as ftp, open(file_path_m, 'rb') as file:
                ftp.storbinary(f'STOR {file_path_m.name}', file)                   
                logging.info(f'{file_path_m} transactions batch uploaded.')
            os.remove(file_path_m)


        except ftplib.error_perm as e:
            logging.error(f"FTP Upload error for {file_path_m}. Re-upload this file in a few minutes.")

    def get_all_transactions(self, acct, start=None, end=None, timeRange=None, status='pending'):
        """GET all transactions over the given time period with a given status and transform
        the transactions into dictionaries consumable by the Impact transaction upload object
        TODO :: Add flexibility for the identifier of new vs returning and device type

        Arguments:
            acct {string} -- Impact program ID
            start {datetime} -- Datetime object for the beginning of the transaction time period
            end {datetime} -- Datetime object for the end of the transaction time period
            status {string} -- Status of the transactions to be pulled - one of [pending, approved, declined]. Defaults to pending if incorrect input is given.

        Returns:
            list -- List of dictionaries containing transaction data
        """       
        transactions = [] # Replace this with whatever network call needs to happen to get a pub list
        url = f'{self.adv_api_stem}Reports/adv_action_listing.json'
    
        if status == 'pending':
            st = 'PENDING'
        elif status == 'approved':
            st = 'APPROVED'
        elif status == 'declined':
            st = 'REVERSED'
        else:
            st = 'PENDING'

        params ={
            'compareEnabled':'false',
            'SUPERSTATUS_MS':st,
            'SUBAID':{self.program_id},
        }

        if start:
            params['START_DATE']=start,
            params['END_DATE']=end,
            params['timeRange']='CUSTOM'
        elif timeRange:
            params['timeRange']=timeRange,
        

        headers = {'Content-Type': 'application/x-www-form-urlencoded', 'Accept':'application/json'}

        while True:
            r = requests.get(url, headers=headers, params=params)
            while r.status_code == 429:
                time.sleep(30)
                r = requests.get(url, headers=headers)
            while r.status_code == 520:
                time.sleep(30)
                r = requests.get(url, headers=headers)
            rj = r.json()

            printProgressBar(int(rj['@page']), int(rj['@numpages']), prefix = f'Page {int(rj["@page"])}/{int(rj["@numpages"])}:', suffix = 'Complete', length = 50)
            time.sleep(2)
            for t in r.json()['Records']:
                transaction = {
                    'transactionDate': t['Action_Date'],
                    'id': t['OID'],
                    'saleAmount': {'amount':t['Sale_Amount'], 'currency':t['Original_Currency']},
                    'commissionAmount': {'amount':t['Payout'], 'currency':t['Original_Currency']},
                    'publisherId': t['MP_Id'],
                    'status': t['Customer_Status'],
                    'voucherCode': t['PromoCode'],
                    'customerCountry': t['CustomerCountry'],
                    'advertiserCountry': t['CustomerCountry']
                }
                if 'Mobile' in t['Action_Tracker']:
                    transaction['device'] = 'Mobile'
                else:
                    transaction['device'] = 'Desktop'
                transactions.append(transaction)
            if rj['@page'] != rj['@numpages']:
                url = self.root_adv_api + rj['@nextpageuri']
            else:
                break

        return transactions


# Helper functions
def pretty_print_POST(req):
    print('{}\n{}\r\n{}\r\n\r\n{}'.format(
        '-----------START-----------',
        req.method + ' ' + req.url,
        '\r\n'.join('{}: {}'.format(k, v) for k, v in req.headers.items()),
        req.body,
    ))

def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end = printEnd)
    # Print New Line on Complete
    if iteration == total: 
        print()

def randomString(stringLength=4):
    import string
    import random
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for i in range(stringLength))

# class Clicker():
#     def __init__(self,tracking_link):
#         self.tracking_link = tracking_link

#         def fire_click(self, mpid, user_agent):
#             to_click = self.tracking_link.format(mpid)
#             print(to_click)
#             req = request.Request(to_click, headers=user_agent)
#             landing_page = request.urlopen(req).geturl()
#             cid = landing_page.split('irclickid=')[1].split('&')[0]
#             print(landing_page)
#             return cid, landing_page

#         def create_clicks(self, account_id, existing_partner_dict):
#             start = (datetime.now() - timedelta(0)).strftime('%Y-%m-%d')
#             end = (datetime.now() - timedelta(0)).strftime('%Y-%m-%d')
#             agg_transactions = get_agg_transactions(account_id, start, end).json()
#             print(agg_transactions)
#             user_agent = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'}

#             for a in agg_transactions:
#                 try:
#                     mpid = existing_partner_dict[str(a['publisherId'])]
#                     clickid, lp = self.fire_click(mpid, user_agent)
#                     time.sleep(.5)
#                     new_clickid, lp = self.fire_click(mpid, user_agent)
#                     logging.info(f'Sending {mpid} clicks - {a["clicks"]}')
#                     for c in range(a['clicks']-1):
#                         time.sleep(.5)
#                         while clickid == new_clickid:
#                             new_clickid, lp = self.fire_click(mpid, user_agent)
#                         printProgressBar(c+1, a['clicks'],
#                                         prefix=f'Sending {mpid} click {c+1} of {a["clicks"]}',
#                                         suffix=f'{lp}',
#                                         length=50)

#                 except KeyError as e:
#                     logging.info(f'{a["publisherId"]} click active but not in program')
    # name = 'Awin_DE'
    # account_id = 'XXXX'
    # threads = []
    # # for name,account_id in account_ids.items():
    #     logging.info(f'Create and start thread for {name} click creation')
    #     x = threading.Thread(target=create_clicks, args=(account_id, existing_partner_dict))
    #     threads.append(x)
    #     x.start()

    # for index, thread in enumerate(threads):
    #     logging.info("Main    : before joining thread %d.", index)
    #     thread.join()
    #     logging.info("Main    : thread %d done", index)


import logging
import os
import csv
from datetime import datetime, timedelta
from pathlib import Path

def new_transaction_lists(impact, t_lists, account_name, network_name):
    """Transforms new transaction records by adding the Impact tracker ID for the environment (desktop/mobile),
        finding the correct Impact partner  ID from the network publisher ID, and determining the commission rate.
        See README for contract payout groups requirements for correct payments.

    Arguments:
        new_transactions {list} -- list of transaction dicts. See below for required key value pairs in the dictionary
        account_name {string} -- Network program/account name
        network_name {string} -- Network name

    Returns:
        tuple -- (list of Impact conversions upload headers, list of amended transaction dicts)
    """
    headrow = ["CampaignId","ActionTrackerId","EventDate","OrderId","MediaPartnerId",'CustomerStatus',"CurrencyCode","Amount","Category","Sku","Quantity",'Text1','PromoCode','Country','OrderLocation','Text2','Date1','Note','Numeric1','OrderStatus','VoucherCode','Modified']
    t_list = []
    for l in t_lists:
        for t in l:
            try:
                mpid = impact.existing_partner_dict[t['publisherId']]
            except KeyError as e:
                try:
                    mpid = impact.existing_partner_dict[str(t['publisherId'])]
                except KeyError as e:
                    logging.warning(f'No valid partner {t["publisherId"]} found for {network_name} transaction {t["id"]}')
                    # mpid = 'NOMPID'
                    continue
            try:
                commission_rate = int(round(float(t['commissionAmount']['amount']) / float(t['saleAmount']['amount']), 2) * 100)
            except ZeroDivisionError as e:
                commission_rate = 0
            if t['device'] == 'Desktop':
                at_id = impact.desktop_action_tracker_id
            elif t['device'] == 'Mobile':
                at_id = impact.mobile_action_tracker_id

            transaction = [
                impact.program_id,
                at_id,
                t['transactionDate'],
                t['id'],
                mpid,
                t['status'],
                t['saleAmount']['currency'],
                t['saleAmount']['amount'],
                'cat',
                'sku',
                1,
                account_name,
                account_name,
                # t['voucherCode'],
                t['customerCountry'],
                t['customerCountry'],
                t['advertiserCountry'],
                t['transactionDate'],
                account_name,
                commission_rate,
                'pending',
                t['voucherCode'],
                f'{datetime.now():%Y-%m-%dT%H:%M:%S}'
                ]
            t_list.append(transaction)  
    return headrow, t_list

def modified_transaction_lists(impact, approved, declined):
    """Transforms modified transaction records by adding the Impact tracker ID for the environment (desktop/mobile) and the Reason Code for the modification.
    Updates the amount in the case of an approval and zeroes the amount in the case of a reversal.

    Arguments:
        approved {list} -- List of approved transactions (dictionaries)
        declined {list} -- List of declined/reversed transactions (dictionaries)

    Returns:
        tuple -- Two items, the Impact modification file standard headers, and the amended list of transactions to be modifed
    """        
    headrow = ['ActionTrackerID','Oid','Amount','Reason']
    t_list = []
    for t in approved:
        if t['device'] == 'Desktop':
            at_id = impact.desktop_action_tracker_id
        elif t['device'] == 'Mobile':
            at_id = impact.mobile_action_tracker_id
        transaction = [
            at_id,
            t['id'],
            t['saleAmount']['amount'],
            'VALIDATED_ORDER'
            ]
        t_list.append(transaction)
    for t in declined:
        if t['device'] == 'Desktop':
            at_id = impact.desktop_action_tracker_id
        elif t['device'] == 'Mobile':
            at_id = impact.mobile_action_tracker_id
        transaction = [
            at_id,
            t['id'],
            0,
            'RETURNED'
            ]
        t_list.append(transaction)

    return headrow, t_list

def prepare_transactions(impact, account_id, network, target_date):
    """Extract transactions from the network object methods (date formats are specific to each network) and pass them to the transformation functions
    TODO :: move the date formatting to the network objects
    TODO :: historical data functions

    Arguments:
        account_id {string} -- Network specific authentication token to get publisher list
        network {object} -- Network object (currently one of AWin, Admitad, Linkshare)
    Keyword Arguments:
        historical {bool} -- Not currently implemented (default: {False})

    Returns:
        tuple -- (list of approved transactions, list of declined, list of pending, end datetime object) (all amended to fit Impact formatting requirements)
    """
    approved = []
    pending = []
    declined = []
    logging.info(f'Getting {network.network_name} transactions and modifications for account {account_id}')
    start,end = network.date_formatter(target_date)

    if network.network_name == 'Linkshare':
        approved, pending, declined = network.get_all_transactions(account_id, start, end)
        declined = []

    else:            
        approved = network.get_all_transactions(account_id, start, end, 'approved')
        declined = network.get_all_transactions(account_id, start, end, 'declined')
        pending = network.get_all_transactions(account_id, start, end, 'pending')

    return approved, declined, pending, target_date

def transactions_process(impact, account_id, account_name, network, target_date):
    """Main function running the extract and transform process for the network transaction data. Writes the results to CSV ready for upload.

    Arguments:
        account_id {string} -- Network account ID
        account_name {string} -- Network program/account name
        network {object} -- Network object (currently one of AWin, Admitad, Linkshare)

    Keyword Arguments:
        historical {bool} -- Not implemented (default: {False})

    Returns:
        file_path_m {string} -- path to the modifications file 
        file_path_p {string} -- path to the pending transactions file 

    """        
    approved, declined, pending, end = prepare_transactions(impact, account_id, network, target_date)
    transactions_filepath = f'transactions/{end.year}/{end:%m}/{end:%d}/{account_name.replace(" ","_")}'
    modifications_filepath = f'modifications/{end.year}/{end:%m}/{end:%d}/{account_name.replace(" ","_")}'

    try:
        os.makedirs(transactions_filepath)
    except FileExistsError as e:
        pass
    
    try:
        os.makedirs(modifications_filepath)

    except FileExistsError as e:
        pass
    file_path_p = Path(f'{transactions_filepath}/{account_name.replace(" ","_")}_{end:%Y-%m-%d}.csv')
    file_path_m = Path(f'{modifications_filepath}/{account_name.replace(" ","_")}_{end:%Y-%m-%d}.csv')

    logging.info(f'Approved transactions {len(approved)}')
    logging.info(f'New pending transactions {len(pending)}')
    logging.info(f'Declined transactions {len(declined)}')

    headrow, t_list = new_transaction_lists(impact, [approved, declined, pending], account_name, network.network_name)
    with open(file_path_p , 'w', newline="") as f:
        csvwriter = csv.writer(f, delimiter = ',')
        csvwriter.writerow(headrow)
        csvwriter.writerows(t_list)

    headrow, t_list = modified_transaction_lists(impact, approved, declined)
    with open(file_path_m , 'w', newline="") as f:
        csvwriter = csv.writer(f, delimiter = ',')
        csvwriter.writerow(headrow)
        csvwriter.writerows(t_list)

    return file_path_m, file_path_p


from setuptools import setup, find_packages

setup(
    name='netimpact',
    version='0.2.3',
    packages=find_packages(),
    install_requires=[
        'Click',
        'requests',
        'toml',
        'boto3',
        's3fs',
        'pandas',
        'pyarrow'
    ],
    entry_points='''
        [console_scripts]
        netimpact=netimpact.netimpact:cli
    ''',
    description='Cross affiliate network data and partner tools',
    url='http://github.com/jalexspringer/netimpact',
    author='Alex Springer',
    author_email='alexspringer@pm.me',
    license='MIT',
)